﻿
Structure DebugHelper_Data
  Map MemMap.i()
  Map MutexMap.i()
EndStructure

Global DebugHelper.DebugHelper_Data

Procedure DebugHelper_Alloc(MemSize)
  Protected Mem
  Shared DebugHelper
  
  Mem = AllocateMemory(MemSize)
  DebugHelper\MemMap(Str(Mem)) = MemSize
  
  ProcedureReturn Mem
EndProcedure

Procedure DebugHelper_ReAlloc(Mem, MemSize)
  Shared DebugHelper
  
  If (DebugHelper\MemMap(Str(Mem)) = #Null) And (Mem <> 0)
    MessageRequester("", "REALOC ERROR")  
  EndIf
  
  DebugHelper\MemMap(Str(Mem)) = #Null
  Mem = ReAllocateMemory(Mem, MemSize)
  DebugHelper\MemMap(Str(Mem)) = MemSize
  
  ProcedureReturn Mem
EndProcedure

Procedure DebugHelper_Free(Mem)
  Shared DebugHelper
  If Not DebugHelper\MemMap(Str(Mem))
    ProcedureReturn #True
  Else
    DebugHelper\MemMap(Str(Mem)) = #Null
  EndIf
  
  FreeMemory(Mem)
EndProcedure


Procedure DebugHelper_CreateMutex()
  Shared DebugHelper
  Protected Mutex
  
  Mutex = CreateMutex()
  DebugHelper\MutexMap(Str(Mutex)) = #True
  
  ProcedureReturn Mutex
EndProcedure

Procedure DebugHelper_FreeMutex(Mutex)
  Shared DebugHelper
  
  If Not DebugHelper\MutexMap(Str(Mutex))
    ProcedureReturn #True
  Else
    DebugHelper\MutexMap(Str(Mutex)) = #Null
  EndIf
  
  FreeMutex(Mutex)
EndProcedure

Procedure DebugHelper_LockMutex(Mutex)
  Shared DebugHelper
  
  If Not DebugHelper\MutexMap(Str(Mutex))
    MessageRequester("", "UNGÜLTIGER MUTEXLOCK!" + Chr(13) + Str(MUTEX))  
  EndIf
  
  ProcedureReturn LockMutex(Mutex)
EndProcedure

Procedure DebugHelper_UnlockMutex(Mutex)
  Shared DebugHelper
  
  
  If Not DebugHelper\MutexMap(Str(Mutex))
    ProcedureReturn #True
  Else
    UnlockMutex(Mutex)
  EndIf
  
  
EndProcedure

Macro AllocateMemory(MemSize)
  DebugHelper_Alloc(MemSize)  
EndMacro

Macro ReAllocateMemory(Mem, MemSize)
  DebugHelper_ReAlloc(Mem, MemSize)  
EndMacro

Macro FreeMemory(Mem)
  If DebugHelper_Free(Mem)  
    MessageRequester("", "UNGÜLTIGE SPEICHERFREIGABE!"+Chr(13) + Str(Mem) + Chr(13) + #PB_Compiler_Procedure+"()/"+Str(#PB_Compiler_Line))  
  EndIf
EndMacro

Macro CreateMutex()
  DebugHelper_CreateMutex()  
EndMacro

Macro LockMutex(Mutex)
  DebugHelper_LockMutex(Mutex)  
EndMacro

Macro UnlockMutex(Mutex)
  If DebugHelper_UnlockMutex(Mutex)
    MessageRequester("", "UNGÜLTIGER MUTEXUNLOCK!" + Chr(13) + Str(MUTEX) + Chr(13) + #PB_Compiler_Procedure+"()/"+Str(#PB_Compiler_Line))  
  EndIf
EndMacro

Macro FreeMutex(Mutex)
 If DebugHelper_FreeMutex(Mutex)  
   MessageRequester("", "UNGÜLTIGE MUTEXFREIGABE!" + Chr(13) + Str(Mutex) + Chr(13) + #PB_Compiler_Procedure+"()/"+Str(#PB_Compiler_Line))  
 EndIf
EndMacro
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 22
; Folding = ---
; EnableXP
; CPU = 1