﻿
XIncludeFile "EProxy_Header.pbi"

EnableExplicit

InitNetwork()

Macro _SOCKS_Receive(_Mem, _MemSize)
  t = ElapsedMilliseconds()
  OverTime = #False
  Repeat
    Delay(5)
    If NetworkClientEvent(ConId) = #PB_NetworkEvent_Data
      ReceiveNetworkData(ConId, _Mem, _MemSize)
      Break
    EndIf
    If (ElapsedMilliseconds()-t) >= timeout
      OverTime = #True
    EndIf
  Until OverTime
EndMacro

; Verbindung über einen SOCKS54 Proxy herstellen
Procedure SOCKS4_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$ = "", timeout = 5000)
  Protected ConId, CmdSize, *Cmd.SOCKS_V4_Ver, t, OverTime, RtVar
  
  ; Verbindung zum Proxy herstellen
  ConId = OpenNetworkConnection(proxy_host$, proxy_port)
  
  If ConId
    CmdSize = SizeOf(SOCKS_V4_Ver) + SizeOf(User$) + SizeOf(Byte)
    *Cmd = AllocateMemory(CmdSize)
    If isIP(dest_host$)
      *Cmd\ver = $04 ; Socks V4
      *Cmd\rsv = 1 ; ?
      *Cmd\port = Port2Octed(dest_port) ; Port
      ; IP
      *Cmd\ip = MakeIPAddress(Val(StringField(dest_host$, 1, ".")), Val(StringField(dest_host$, 2, ".")), Val(StringField(dest_host$, 3, ".")), Val(StringField(dest_host$, 4, ".")))
      ; User
      PokeS(*Cmd + SizeOf(SOCKS_V4_Ver), User$)
      
      SendNetworkData(ConId, *Cmd, CmdSize) ; Connect senden
      _SOCKS_Receive(*CMD, CmdSize) ; Auf Antwort warten
  
      
      If (Not OverTime) And *Cmd\ver = #Null And *Cmd\rsv = #SOCKS_V4_REP_Succeeded ; Alles ok?
        RtVar = ConId ; Verbindung hergstellt
      Else
        CloseNetworkConnection(ConId)
      EndIf
    EndIf
    FreeMemory(*CMD)
  EndIf
  
  ProcedureReturn RtVar
EndProcedure

; Verbindung über einen SOCKS5 Proxy herstellen
Procedure SOCKS5_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$ = "", Pass$ = "", timeout = 5000)
  Protected ConId, Ver.SOCKS_Ver, *Cmd.SOCKS_Cmd, t, RtVar, CmdSize, Repl.SOCKS_Reply, OverTime
  
  ; Verbindung zum Proxy herstellen
  ConId = OpenNetworkConnection(proxy_host$, proxy_port)
  
  If ConId
    CmdSize = StringByteLength(User$) + StringByteLength(Pass$) + SizeOf(SOCKS_Cmd)
    *Cmd = AllocateMemory(CmdSize)
    
    Ver\Ver = #SOCKS_Version
    
    ; Phase 1: Anmelden
    If Not User$ ; Kein User, Offener Proxy
      Ver\NMethods = 1
      Ver\Methods[0] = #SOCKS_Method_NoAuthenticationRequired
    Else ; Offen oder Auth
      Ver\NMethods = 2
      Ver\Methods[0] = #SOCKS_Method_NoAuthenticationRequired
      Ver\Methods[1] = #SOCKS_Method_UserNamePassword
    EndIf
    
    SendNetworkData(ConId, @Ver, SizeOf(Word) + Ver\NMethods*SizeOf(Byte))
    _SOCKS_Receive(@Ver, SizeOf(Word)) ; Auf Antwort warten
    
    If Not (OverTime Or Ver\Ver <> #SOCKS_Version) ; alles Ok bisher
      Select Ver\Method
        Case #SOCKS_Method_NoAuthenticationRequired
          Ver\Method = #SOCKS_AUTH_Ok
          ; Nichts mehr zu tun, der Server verbindet ohne Passwort ...  
          
        Case #SOCKS_Method_UserNamePassword ; Der Server will ein Passwort
          ; Phase 2: Auth
          *Cmd\ver = #SOCKS_Version
          *Cmd\cmd = StringByteLength(User$)
          CopyMemory(@User$, *Cmd + 2*SizeOf(Byte), StringByteLength(User$))
          PokeA(*Cmd + 2*SizeOf(Byte) + StringByteLength(User$), StringByteLength(Pass$))
          CopyMemory(@Pass$, *Cmd + 2*SizeOf(Byte) + StringByteLength(User$), StringByteLength(Pass$))
          
          SendNetworkData(ConId, *Cmd, CmdSize) ; Auth senden
          _SOCKS_Receive(@Ver, SizeOf(Word)) ; Auf Antwort warten
          
          If (OverTime Or Ver\ver <> #SOCKS_Version) ; Fehler
            Ver\Method = #SOCKS_AUTH_FAILURE
          EndIf
          
      EndSelect
      
      If Ver\Method = #SOCKS_AUTH_Ok ; Jetzt kommen wir zum Connect
        ; Wir führen einfach eine Verbindung per DomainName durch, dadurch umgehen wir auch DNS - Filter
        CmdSize = SizeOf(SOCKS_Cmd) + 3*SizeOf(Byte) + StringByteLength(dest_host$)
        *CMD = ReAllocateMemory(*CMD, CmdSize)
        
        *CMD\ver = #SOCKS_Version
        *CMD\cmd = #SOCKS_CMD_Connect
        *CMD\rsv = #Null
        *CMD\atyp = #SOCKS_ATYP_DomainName
        PokeA(*CMD + SizeOf(SOCKS_Cmd), StringByteLength(dest_host$))
        CopyMemory(@dest_host$, *CMD + SizeOf(SOCKS_Cmd) + SizeOf(Byte), StringByteLength(dest_host$))
        PokeW(*CMD + SizeOf(SOCKS_Cmd) + SizeOf(Byte) + StringByteLength(dest_host$), Port2Octed(dest_port))
        
        SendNetworkData(ConId, *Cmd, CmdSize) ; Connect senden
        _SOCKS_Receive(@Repl, SizeOf(SOCKS_Reply)) ; Auf Antwort warten
        
        If Not (OverTime Or Repl\Ver <> #SOCKS_Version)
          If Repl\rep = #SOCKS_REP_Succeeded
            RtVar = #True
          EndIf
        EndIf
      EndIf
    EndIf
    
    FreeMemory(*Cmd)
    If Not RtVar
      CloseNetworkConnection(ConId)
      ConId = #Null
    Else
      RtVar = ConId
    EndIf
  EndIf
    
  ProcedureReturn RtVar
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 119
; FirstLine = 101
; Folding = -
; EnableXP
; CPU = 1