﻿; ------------------------------------------------------------------------------------
; Konstanten
; ------------------------------------------------------------------------------------

; Version
#SOCKS_Version = $05

; Methoden
; $03 - $7F IANA Assigned
; $80 - $FE Reserved for Private Methodes
#SOCKS_Method_NoAuthenticationRequired = $00
#SOCKS_Method_GSSAPI = $01
#SOCKS_Method_UserNamePassword = $02
#SOCKS_Method_NoAcceptableMethod = $00

; Socks Kommandos
#SOCKS_CMD_Connect = $01
#SOCKS_CMD_Bind = $02
#SOCKS_CMD_UDPAssociate = $03

; Adresstype
#SOCKS_ATYP_IPv4 = $01
#SOCKS_ATYP_DomainName = $03
#SOCKS_ATYP_IPv6 = $04
#SOCKS_ATYP_SOCKS4 = $05

; Antworten
; $09 - $FF IANA Assigned
#SOCKS_V4_REP_Succeeded = $5A
#SOCKS_REP_Succeeded = $00
#SOCKS_REP_GeneralSocksServerFailure = $01
#SOCKS_REP_ConnectionNotAllowedByRuleset = $02
#SOCKS_REP_NetworkUnreachable = $03
#SOCKS_REP_HostUnreachable = $04
#SOCKS_REP_ConnectionRefused = $05
#SOCKS_REP_TTLExpired = $06
#SOCKS_REP_CommandNotSupported = $07
#SOCKS_REP_AddressTypeNotSupported = $08

; Authentication
#SOCKS_AUTH_Ok = $00
#SOCKS_AUTH_Failure = $01

#Client_Step_Ver = $00
#Client_Step_Auth = $01
#Client_Step_GSSAPI = $02
#Client_Step_Cmd = $03
#Client_Step_Transmit = $04

; ------------------------------------------------------------------------------------
; Strukturen
; ------------------------------------------------------------------------------------

; Nur für die Statitik ...
Structure EProxy_Statistics
  TotalDownloadedBytes.q
  TotalUploadedBytes.q
  TotalConnections.q
  LastDownloadedBytes.q
  LastUploadedBytes.q
  DownloadRate.d
  UploadRate.d
  DownloadTimeStamp.i
  UploadTimeStamp.i
  TDBMutex.i
  TUBMutex.i
  TCMutex.i
EndStructure

; ProxyDaten
Structure EProxy_Data
  Server.i ; Server ID
  ReceiveBlockSize.i ; Empfangsblockgröße
  TransmitterReceiveBlockSize.i ; Transmitter Empfangsblockgröße
  Stop.i ; Stop
  Clients.i ; Client - Index
  *ClientList.LL_Data ; Client - Liste
  ClientManager.i ; Client - Prozessor
  LastClient.i ; Erkennung für Doppeltes beenden eines Client
  ClientMutex.i ; ClientMutex
  *Worker ; Arbeiter - Thread
  NeedAuth.i ; Benötige Auth
  Statistic.EProxy_Statistics ; Statistik.
EndStructure

; Erste Anfrage
Structure SOCKS_Ver
  ver.a ; Version, muss $05 sein
  StructureUnion
    nMethods.a ; Anzahl methoden
    Method.a ; Methode
  EndStructureUnion
  Methods.a[255] ; Auswahl an Methoden
EndStructure

Structure SOCKS_V4_Ver
  ver.a
  rsv.a
  port.w
  ip.l
EndStructure

; Socks CMD
Structure SOCKS_Cmd
  ver.a ; Version, muss $05 sein
  cmd.a ; Kommando
  rsv.a ; Reserved muss $00 sein
  atyp.a ; Addresstyp
EndStructure

; Socks Reply
Structure SOCKS_Reply
  ver.a ; Version, muss $05 sein
  rep.a ; Antwort
  rsv.a ; Reserved muss $00 sein
  atyp.a ; Addresstyp
  addr.l ; Addresse
  port.w ; Port
EndStructure

; Socks Adresse
Structure SOCKS_Addr
  Addr.l
  Port.w
EndStructure

; Commando Prozessor
Structure SOCKS_CMDProcess
  EProxy.i
  Client.i
  *Mem
  MemSize.i
EndStructure

; Octed - Struktur
Structure _SOCKS_Port_Oct
  high.a
  low.a
EndStructure

; Proxy Data - Transmitter
Structure EProxy_Transmitter
  *EProxy.EProxy_Data ; ECluster Proxy
  Client.i ; Client
  Connection.i ; Verbindung
  Stop.i ; Beenden
  SuicideDone.i ; Suicide
EndStructure

; Proxy User Data
Structure EProxy_UserData
  State.i  ; Verbindungsstatus
  Connection.i ; Weiterleitung
  *Transmitter.EProxy_Transmitter ; Transmitter Thread
  *LLEntry.i ; LinkedList eintrag
  SendSlot.i ; SendeSlot
  PMutex.i ; Schutz - Mutex für kritische Teile
  Removing.a ; Client wird entfernt
EndStructure

; Sendestruktur
Structure EProxy_Sender
  Client.i ; Client von dem die Verbindung ausgeht
  Connection.i ; Verbindung
  *Mem ; Speicher
  MemSize.i ; Speichergröße
  *EProxy.EProxy_Data ; EProxy Handle
EndStructure

; UserManager
Structure EProxy_UserManager
  *EProxy
  Client.i
  Suicide.a
  *User
EndStructure

; ------------------------------------------------------------------------------------
; Proceduren
; ------------------------------------------------------------------------------------

; Port in Octed - Schreibweise
Procedure.w Port2Octed(Port)
  Protected Oct.w
  Protected t8Port._SOCKS_Port_Oct
    
  t8Port\high = Int(Port/256)
  t8Port\low = Port-(256*t8Port\high)
  Oct = PeekW(@t8Port)
  
  ProcedureReturn Oct
EndProcedure

; Octed in Port Schreibweise
Procedure Octed2Port(Oct)
  Protected *t8Port._SOCKS_Port_Oct
  Protected Port
  
  *t8Port = @Oct
  Port = *t8Port\high*256 + *t8Port\low
  
  ProcedureReturn Port
EndProcedure

Procedure IsIP(ip.s)
  Protected count, ret, i, a$, va
  ;ReturnValues :
  ;0 = no ip
  ;1 = IPv4 - IP
  count = CountString(ip, ".")
  If count = 3
    ret = 1
  Else
    ProcedureReturn 0
  EndIf
  For i = 0 To count
    a$ = StringField(ip, i + 1, ".")
    va = Val(a$)
    If a$ = "" Or Str(va)<>a$ Or va>255 Or va<0
      ProcedureReturn 0
    EndIf
  Next i
  ProcedureReturn ret
EndProcedure


; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 156
; FirstLine = 144
; Folding = -
; EnableXP
; CPU = 1
; DisableDebugger