﻿; ------------------------------------------------------------------------------------
; Sichert die PB - Netzwerkbefehle ab
; ------------------------------------------------------------------------------------

; Kommentieren

EnableExplicit

XIncludeFile "NCEDisconnect.pbi"

Structure __ConnectionIDCollector_Data
  Map Connections.i()
  Mutex.i
EndStructure

Global __ConnectionIDCollector.__ConnectionIDCollector_Data

; Erstellt die Verwaltung
Procedure __ConnectionIDCollector_Init()
  Shared __ConnectionIDCollector
  With __ConnectionIDCollector
    \Mutex = CreateMutex()
  EndWith
EndProcedure : __ConnectionIDCollector_Init()

Procedure __NetworkServerEvent()
  Protected Event, Mutex, EventClient
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    Event = NetworkServerEvent()
    
    If Event = #PB_NetworkEvent_Connect
      
      LockMutex(\Mutex)
      
      EventClient = EventClient()
      If Not \Connections(Str(EventClient))
        \Connections() = CreateMutex()
      EndIf
      
      UnlockMutex(\Mutex)
      
    ElseIf Event = #PB_NetworkEvent_Disconnect
      
      LockMutex(\Mutex)
      
      Mutex = \Connections(Str(EventClient()))
      If Mutex
        \Connections() = #Null
        DeleteMapElement(\Connections())
        
        LockMutex(Mutex)
        UnlockMutex(Mutex)
        FreeMutex(Mutex)
      EndIf
      
      UnlockMutex(\Mutex)
      
    EndIf
  EndWith
  
  ProcedureReturn Event
EndProcedure

Procedure __NetworkClientEvent(Connection)
  Protected Event, Mutex
  Protected hSocket.i, tv.timeval, readfds.fd_set, Length.i
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    
    LockMutex(\Mutex)
    Mutex = \Connections(Str(Connection))
    UnlockMutex(\Mutex)
    
    If Mutex
      LockMutex(Mutex)
      Event = NetworkClientEvent(Connection)
      If Not Event
        
        hSocket = ConnectionID(Connection)
        tv\tv_sec  = 0 ; Dont even wait, just query status
        tv\tv_usec = 0
      
        If hSocket > 65536 Or hSocket < 1 ; Gültiger bereich
          Event = #PB_NetworkEvent_Disconnect
        Else
      
          FD_ZERO(readfds)
          FD_SET(hSocket, readfds)
      
          ; Check if there is something new
          Event = select_(_NFDS(readfds), @readfds, #Null, #Null, @tv)
          If Event < 0 ; Seems to be an error
            Event = #PB_NetworkEvent_Disconnect
          ElseIf Event = 0 Or Not FD_ISSET(hSocket, readfds) ; No data available
            Event = #Null
          Else
             ; Check if data is available?
            CompilerIf #PB_Compiler_OS = #PB_OS_Windows
              Event = ioctlsocket_(hSocket, #FIONREAD, @Length)
            CompilerElse
              Event = ioctl_(hSocket, #FIONREAD, @Length)
            CompilerEndIf
      
            If Event Or Length = 0 ; Not successful to query for data available OR no data available ? This seems to be an error!
              Event = #PB_NetworkEvent_Disconnect
            Else
              Event = #Null
            EndIf
          EndIf
        EndIf
      EndIf
      
      UnlockMutex(Mutex)

      If Event = #PB_NetworkEvent_Disconnect
        LockMutex(\Mutex)
        If \Connections(Str(Connection))
          \Connections() = #Null
          DeleteMapElement(\Connections())
          LockMutex(Mutex)
          UnlockMutex(Mutex)
          FreeMutex(Mutex)
        EndIf
        UnlockMutex(\Mutex)
      EndIf
    Else
      Event = #PB_NetworkEvent_Disconnect
    EndIf
    
  EndWith
  
  ProcedureReturn Event
EndProcedure

Procedure __OpenNetworkConnection(Host$, Port, Type = #PB_Network_TCP)
  Protected Connection
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    Connection = OpenNetworkConnection(Host$, Port, Type)
    LockMutex(\Mutex)
    \Connections(Str(Connection)) = CreateMutex()
    UnlockMutex(\Mutex)
  EndWith
  
  ProcedureReturn Connection
EndProcedure

Procedure __CloseNetworkConnection(Connection)
  Protected isOk, Mutex
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    LockMutex(\Mutex)
    Mutex = \Connections(Str(Connection))
    If Mutex
      \Connections() = #Null
      DeleteMapElement(\Connections())
    EndIf
    UnlockMutex(\Mutex)
    If Mutex
      LockMutex(Mutex)
      isOk = CloseNetworkConnection(Connection)  
      UnlockMutex(Mutex)
      ; Freigeben
      LockMutex(Mutex)
      UnlockMutex(Mutex)
      FreeMutex(Mutex)
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __ConnectionID(Connection)
  Protected isOk, Mutex
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    LockMutex(\Mutex)
    Mutex = \Connections(Str(Connection))
    UnlockMutex(\Mutex)
    If Mutex
      LockMutex(Mutex)
      isOk = ConnectionID(Connection)
      UnlockMutex(Mutex)
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __SendNetworkData(Connection, *Mem, MemSize)
  Protected isOk, Mutex
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    LockMutex(\Mutex)
    Mutex = \Connections(Str(Connection))
    UnlockMutex(\Mutex)
    If Mutex
      LockMutex(Mutex)
      isOk = SendNetworkData(Connection, *Mem, MemSize)
      UnlockMutex(Mutex)
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __ReceiveNetworkData(Connection, *Mem, MemSize)
  Protected isOk, Mutex
  Shared __ConnectionIDCollector
  
  With __ConnectionIDCollector
    LockMutex(\Mutex)
    Mutex = \Connections(Str(Connection))
    UnlockMutex(\Mutex)
    If Mutex
      LockMutex(Mutex)
      isOk = ReceiveNetworkData(Connection, *Mem, MemSize)
      UnlockMutex(Mutex)
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

; PB - Befehle überschreiben

Macro NetworkServerEvent()
  __NetworkServerEvent()  
EndMacro

Macro NetworkClientEvent(Connection)
  __NetworkClientEvent(Connection)  
EndMacro

Macro OpenNetworkConnection(ServerName, Port, Type = #PB_Network_TCP)
  __OpenNetworkConnection(ServerName, Port, Type)  
EndMacro

Macro CloseNetworkConnection(Connection)
  __CloseNetworkConnection(Connection)  
EndMacro

Macro ConnectionID(Connection)
  __ConnectionID(Connection)  
EndMacro

Macro SendNetworkData(Connection, Mem, MemSize)
  __SendNetworkData(Connection, Mem, MemSize)  
EndMacro

Macro ReceiveNetworkData(Connection, Mem, MemSize)
  __ReceiveNetworkData(Connection, Mem, MemSize)
EndMacro

; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 222
; Folding = ---
; EnableXP
; CPU = 1